# Copyright Iris contributors
#
# This file is part of Iris and is released under the LGPL license.
# See COPYING and COPYING.LESSER in the root of the repository for full
# licensing details.
"""
Unit tests for the
`iris.aux_factory.HybridPressureFactory` class.

"""

# Import iris.tests first so that some things can be initialised before
# importing anything else.
import iris.tests as tests

from unittest import mock

import cf_units
import numpy as np

import iris
from iris.aux_factory import HybridPressureFactory


class Test___init__(tests.IrisTest):
    def setUp(self):
        self.delta = mock.Mock(units=cf_units.Unit("Pa"), nbounds=0)
        self.sigma = mock.Mock(units=cf_units.Unit("1"), nbounds=0)
        self.surface_air_pressure = mock.Mock(
            units=cf_units.Unit("Pa"), nbounds=0
        )

    def test_insufficient_coords(self):
        with self.assertRaises(ValueError):
            HybridPressureFactory()
        with self.assertRaises(ValueError):
            HybridPressureFactory(
                delta=None, sigma=self.sigma, surface_air_pressure=None
            )
        with self.assertRaises(ValueError):
            HybridPressureFactory(
                delta=None,
                sigma=None,
                surface_air_pressure=self.surface_air_pressure,
            )

    def test_incompatible_delta_units(self):
        self.delta.units = cf_units.Unit("m")
        with self.assertRaises(ValueError):
            HybridPressureFactory(
                delta=self.delta,
                sigma=self.sigma,
                surface_air_pressure=self.surface_air_pressure,
            )

    def test_incompatible_sigma_units(self):
        self.sigma.units = cf_units.Unit("Pa")
        with self.assertRaises(ValueError):
            HybridPressureFactory(
                delta=self.delta,
                sigma=self.sigma,
                surface_air_pressure=self.surface_air_pressure,
            )

    def test_incompatible_surface_air_pressure_units(self):
        self.surface_air_pressure.units = cf_units.Unit("unknown")
        with self.assertRaises(ValueError):
            HybridPressureFactory(
                delta=self.delta,
                sigma=self.sigma,
                surface_air_pressure=self.surface_air_pressure,
            )

    def test_different_pressure_units(self):
        self.delta.units = cf_units.Unit("hPa")
        self.surface_air_pressure.units = cf_units.Unit("Pa")
        with self.assertRaises(ValueError):
            HybridPressureFactory(
                delta=self.delta,
                sigma=self.sigma,
                surface_air_pressure=self.surface_air_pressure,
            )

    def test_too_many_delta_bounds(self):
        self.delta.nbounds = 4
        with self.assertRaises(ValueError):
            HybridPressureFactory(
                delta=self.delta,
                sigma=self.sigma,
                surface_air_pressure=self.surface_air_pressure,
            )

    def test_too_many_sigma_bounds(self):
        self.sigma.nbounds = 4
        with self.assertRaises(ValueError):
            HybridPressureFactory(
                delta=self.delta,
                sigma=self.sigma,
                surface_air_pressure=self.surface_air_pressure,
            )

    def test_factory_metadata(self):
        factory = HybridPressureFactory(
            delta=self.delta,
            sigma=self.sigma,
            surface_air_pressure=self.surface_air_pressure,
        )
        self.assertEqual(factory.standard_name, "air_pressure")
        self.assertIsNone(factory.long_name)
        self.assertIsNone(factory.var_name)
        self.assertEqual(factory.units, self.delta.units)
        self.assertEqual(factory.units, self.surface_air_pressure.units)
        self.assertIsNone(factory.coord_system)
        self.assertEqual(factory.attributes, {})

    def test_promote_sigma_units_unknown_to_dimensionless(self):
        sigma = mock.Mock(units=cf_units.Unit("unknown"), nbounds=0)
        factory = HybridPressureFactory(
            delta=self.delta,
            sigma=sigma,
            surface_air_pressure=self.surface_air_pressure,
        )
        self.assertEqual("1", factory.dependencies["sigma"].units)


class Test_dependencies(tests.IrisTest):
    def setUp(self):
        self.delta = mock.Mock(units=cf_units.Unit("Pa"), nbounds=0)
        self.sigma = mock.Mock(units=cf_units.Unit("1"), nbounds=0)
        self.surface_air_pressure = mock.Mock(
            units=cf_units.Unit("Pa"), nbounds=0
        )

    def test_value(self):
        kwargs = dict(
            delta=self.delta,
            sigma=self.sigma,
            surface_air_pressure=self.surface_air_pressure,
        )
        factory = HybridPressureFactory(**kwargs)
        self.assertEqual(factory.dependencies, kwargs)


class Test_make_coord(tests.IrisTest):
    @staticmethod
    def coords_dims_func(coord):
        mapping = dict(
            level_pressure=(0,), sigma=(0,), surface_air_pressure=(1, 2)
        )
        return mapping[coord.name()]

    def setUp(self):
        self.delta = iris.coords.DimCoord(
            [0.0, 1.0, 2.0], long_name="level_pressure", units="Pa"
        )
        self.sigma = iris.coords.DimCoord(
            [1.0, 0.9, 0.8], long_name="sigma", units="1"
        )
        self.surface_air_pressure = iris.coords.AuxCoord(
            np.arange(4).reshape(2, 2), "surface_air_pressure", units="Pa"
        )

    def test_points_only(self):
        # Determine expected coord by manually broadcasting coord points
        # knowing the dimension mapping.
        delta_pts = self.delta.points[..., np.newaxis, np.newaxis]
        sigma_pts = self.sigma.points[..., np.newaxis, np.newaxis]
        surf_pts = self.surface_air_pressure.points[np.newaxis, ...]
        expected_points = delta_pts + sigma_pts * surf_pts
        expected_coord = iris.coords.AuxCoord(
            expected_points, standard_name="air_pressure", units="Pa"
        )
        factory = HybridPressureFactory(
            delta=self.delta,
            sigma=self.sigma,
            surface_air_pressure=self.surface_air_pressure,
        )
        derived_coord = factory.make_coord(self.coords_dims_func)
        self.assertEqual(expected_coord, derived_coord)

    def test_none_delta(self):
        delta_pts = 0
        sigma_pts = self.sigma.points[..., np.newaxis, np.newaxis]
        surf_pts = self.surface_air_pressure.points[np.newaxis, ...]
        expected_points = delta_pts + sigma_pts * surf_pts
        expected_coord = iris.coords.AuxCoord(
            expected_points, standard_name="air_pressure", units="Pa"
        )
        factory = HybridPressureFactory(
            sigma=self.sigma, surface_air_pressure=self.surface_air_pressure
        )
        derived_coord = factory.make_coord(self.coords_dims_func)
        self.assertEqual(expected_coord, derived_coord)

    def test_none_sigma(self):
        delta_pts = self.delta.points[..., np.newaxis, np.newaxis]
        sigma_pts = 0
        surf_pts = self.surface_air_pressure.points[np.newaxis, ...]
        expected_points = delta_pts + sigma_pts * surf_pts
        expected_coord = iris.coords.AuxCoord(
            expected_points, standard_name="air_pressure", units="Pa"
        )
        factory = HybridPressureFactory(
            delta=self.delta, surface_air_pressure=self.surface_air_pressure
        )
        derived_coord = factory.make_coord(self.coords_dims_func)
        self.assertEqual(expected_coord, derived_coord)

    def test_none_surface_air_pressure(self):
        # Note absence of broadcasting as multidimensional coord
        # is not present.
        expected_points = self.delta.points
        expected_coord = iris.coords.AuxCoord(
            expected_points, standard_name="air_pressure", units="Pa"
        )
        factory = HybridPressureFactory(delta=self.delta, sigma=self.sigma)
        derived_coord = factory.make_coord(self.coords_dims_func)
        self.assertEqual(expected_coord, derived_coord)

    def test_with_bounds(self):
        self.delta.guess_bounds(0)
        self.sigma.guess_bounds(0.5)
        # Determine expected coord by manually broadcasting coord points
        # and bounds based on the dimension mapping.
        delta_pts = self.delta.points[..., np.newaxis, np.newaxis]
        sigma_pts = self.sigma.points[..., np.newaxis, np.newaxis]
        surf_pts = self.surface_air_pressure.points[np.newaxis, ...]
        expected_points = delta_pts + sigma_pts * surf_pts
        delta_vals = self.delta.bounds.reshape(3, 1, 1, 2)
        sigma_vals = self.sigma.bounds.reshape(3, 1, 1, 2)
        surf_vals = self.surface_air_pressure.points.reshape(1, 2, 2, 1)
        expected_bounds = delta_vals + sigma_vals * surf_vals
        expected_coord = iris.coords.AuxCoord(
            expected_points,
            standard_name="air_pressure",
            units="Pa",
            bounds=expected_bounds,
        )
        factory = HybridPressureFactory(
            delta=self.delta,
            sigma=self.sigma,
            surface_air_pressure=self.surface_air_pressure,
        )
        derived_coord = factory.make_coord(self.coords_dims_func)
        self.assertEqual(expected_coord, derived_coord)


class Test_update(tests.IrisTest):
    def setUp(self):
        self.delta = mock.Mock(units=cf_units.Unit("Pa"), nbounds=0)
        self.sigma = mock.Mock(units=cf_units.Unit("1"), nbounds=0)
        self.surface_air_pressure = mock.Mock(
            units=cf_units.Unit("Pa"), nbounds=0
        )

        self.factory = HybridPressureFactory(
            delta=self.delta,
            sigma=self.sigma,
            surface_air_pressure=self.surface_air_pressure,
        )

    def test_good_delta(self):
        new_delta_coord = mock.Mock(units=cf_units.Unit("Pa"), nbounds=0)
        self.factory.update(self.delta, new_delta_coord)
        self.assertIs(self.factory.delta, new_delta_coord)

    def test_bad_delta(self):
        new_delta_coord = mock.Mock(units=cf_units.Unit("1"), nbounds=0)
        with self.assertRaises(ValueError):
            self.factory.update(self.delta, new_delta_coord)

    def test_alternative_bad_delta(self):
        new_delta_coord = mock.Mock(units=cf_units.Unit("Pa"), nbounds=4)
        with self.assertRaises(ValueError):
            self.factory.update(self.delta, new_delta_coord)

    def test_good_surface_air_pressure(self):
        new_surface_p_coord = mock.Mock(units=cf_units.Unit("Pa"), nbounds=0)
        self.factory.update(self.surface_air_pressure, new_surface_p_coord)
        self.assertIs(self.factory.surface_air_pressure, new_surface_p_coord)

    def test_bad_surface_air_pressure(self):
        new_surface_p_coord = mock.Mock(units=cf_units.Unit("km"), nbounds=0)
        with self.assertRaises(ValueError):
            self.factory.update(self.surface_air_pressure, new_surface_p_coord)

    def test_non_dependency(self):
        old_coord = mock.Mock()
        new_coord = mock.Mock()
        orig_dependencies = self.factory.dependencies
        self.factory.update(old_coord, new_coord)
        self.assertEqual(orig_dependencies, self.factory.dependencies)

    def test_none_delta(self):
        self.factory.update(self.delta, None)
        self.assertIsNone(self.factory.delta)

    def test_none_sigma(self):
        self.factory.update(self.sigma, None)
        self.assertIsNone(self.factory.sigma)

    def test_insufficient_coords(self):
        self.factory.update(self.delta, None)
        with self.assertRaises(ValueError):
            self.factory.update(self.surface_air_pressure, None)


if __name__ == "__main__":
    tests.main()



# Copyright Iris contributors
#
# This file is part of Iris and is released under the LGPL license.
# See COPYING and COPYING.LESSER in the root of the repository for full
# licensing details.
"""
Test interaction between :mod:`iris.plot` and
:func:`matplotlib.pyplot.colorbar`

"""

# import iris tests first so that some things can be initialised before
# importing anything else
import iris.tests as tests

import numpy as np

from iris.coords import AuxCoord
import iris.tests.stock

# Run tests in no graphics mode if matplotlib is not available.
if tests.MPL_AVAILABLE:
    import matplotlib.pyplot as plt
    from iris.plot import (
        contour,
        contourf,
        pcolormesh,
        pcolor,
        points,
        scatter,
    )


@tests.skip_plot
class TestColorBarCreation(tests.GraphicsTest):
    def setUp(self):
        super().setUp()
        self.draw_functions = (contour, contourf, pcolormesh, pcolor)
        self.cube = iris.tests.stock.lat_lon_cube()
        self.cube.coord("longitude").guess_bounds()
        self.cube.coord("latitude").guess_bounds()
        self.traj_lon = AuxCoord(
            np.linspace(-180, 180, 50),
            standard_name="longitude",
            units="degrees",
        )
        self.traj_lat = AuxCoord(
            np.sin(np.deg2rad(self.traj_lon.points)) * 30.0,
            standard_name="latitude",
            units="degrees",
        )

    def test_common_draw_functions(self):
        for draw_function in self.draw_functions:
            mappable = draw_function(self.cube)
            cbar = plt.colorbar()
            self.assertIs(
                cbar.mappable,
                mappable,
                msg="Problem with draw function iris.plot.{}".format(
                    draw_function.__name__
                ),
            )

    def test_common_draw_functions_specified_mappable(self):
        for draw_function in self.draw_functions:
            mappable_initial = draw_function(self.cube, cmap="cool")
            _ = draw_function(self.cube)
            cbar = plt.colorbar(mappable_initial)
            self.assertIs(
                cbar.mappable,
                mappable_initial,
                msg="Problem with draw function iris.plot.{}".format(
                    draw_function.__name__
                ),
            )

    def test_points_with_c_kwarg(self):
        mappable = points(self.cube, c=self.cube.data)
        cbar = plt.colorbar()
        self.assertIs(cbar.mappable, mappable)

    def test_points_with_c_kwarg_specified_mappable(self):
        mappable_initial = points(self.cube, c=self.cube.data, cmap="cool")
        _ = points(self.cube, c=self.cube.data)
        cbar = plt.colorbar(mappable_initial)
        self.assertIs(cbar.mappable, mappable_initial)

    def test_scatter_with_c_kwarg(self):
        mappable = scatter(
            self.traj_lon, self.traj_lat, c=self.traj_lon.points
        )
        cbar = plt.colorbar()
        self.assertIs(cbar.mappable, mappable)

    def test_scatter_with_c_kwarg_specified_mappable(self):
        mappable_initial = scatter(
            self.traj_lon, self.traj_lat, c=self.traj_lon.points
        )
        _ = scatter(
            self.traj_lon, self.traj_lat, c=self.traj_lon.points, cmap="cool"
        )
        cbar = plt.colorbar(mappable_initial)
        self.assertIs(cbar.mappable, mappable_initial)


if __name__ == "__main__":
    tests.main()



"""
Quickplot of a 2D Cube on a Map
===============================

This example demonstrates a contour plot of global air temperature. The plot
title and the labels for the axes are automatically derived from the metadata.

"""

import cartopy.crs as ccrs
import matplotlib.pyplot as plt

import iris
import iris.plot as iplt
import iris.quickplot as qplt


def main():
    fname = iris.sample_data_path("air_temp.pp")
    temperature = iris.load_cube(fname)

    # Plot #1: contourf with axes longitude from -180 to 180
    plt.figure(figsize=(12, 5))
    plt.subplot(121)
    qplt.contourf(temperature, 15)
    plt.gca().coastlines()

    # Plot #2: contourf with axes longitude from 0 to 360
    proj = ccrs.PlateCarree(central_longitude=-180.0)
    plt.subplot(122, projection=proj)
    qplt.contourf(temperature, 15)
    plt.gca().coastlines()
    iplt.show()


if __name__ == "__main__":
    main()



# Copyright Iris contributors
#
# This file is part of Iris and is released under the LGPL license.
# See COPYING and COPYING.LESSER in the root of the repository for full
# licensing details.
"""
Test function :func:`iris.fileformats._pyke_rules.compiled_krb.\
fc_rules_cf_fc.build_geostationary_coordinate_system`.

"""

# import iris tests first so that some things can be initialised before
# importing anything else
import iris.tests as tests

from unittest import mock

import iris
from iris.coord_systems import Geostationary
from iris.fileformats._pyke_rules.compiled_krb.fc_rules_cf_fc import \
    build_geostationary_coordinate_system


class TestBuildGeostationaryCoordinateSystem(tests.IrisTest):
    def _test(self, inverse_flattening=False, replace_props=None, remove_props=None):
        """
        Generic test that can check vertical perspective validity with or
        without inverse flattening.
        """
        # Make a dictionary of the non-ellipsoid properties to be added to both a test
        # coord-system, and a test grid-mapping cf_var.
        non_ellipsoid_kwargs = {
            'latitude_of_projection_origin': 0.0,
            'longitude_of_projection_origin': 2.0,
            'perspective_point_height': 2000000.0,
            'sweep_angle_axis': 'x',
            'false_easting': 100.0,
            'false_northing': 200.0}

        # Make specified adjustments to the non-ellipsoid properties.
        if remove_props:
            for key in remove_props:
                non_ellipsoid_kwargs.pop(key, None)
        if replace_props:
            for key, value in replace_props.items():
                non_ellipsoid_kwargs[key] = value

        # Make a dictionary of ellipsoid properties, to be added to both a test
        # ellipsoid and the grid-mapping cf_var.
        ellipsoid_kwargs = {'semi_major_axis': 6377563.396}
        if inverse_flattening:
            ellipsoid_kwargs['inverse_flattening'] = 299.3249646
        else:
            ellipsoid_kwargs['semi_minor_axis'] = 6356256.909

        cf_grid_var_kwargs = non_ellipsoid_kwargs.copy()
        cf_grid_var_kwargs.update(ellipsoid_kwargs)
        cf_grid_var = mock.Mock(spec=[], **cf_grid_var_kwargs)
        cs = build_geostationary_coordinate_system(None, cf_grid_var)
        ellipsoid = iris.coord_systems.GeogCS(**ellipsoid_kwargs)
        expected = Geostationary(ellipsoid=ellipsoid, **non_ellipsoid_kwargs)
        self.assertEqual(cs, expected)

    def test_valid(self):
        self._test(inverse_flattening=False)

    def test_inverse_flattening(self):
        self._test(inverse_flattening=True)

    def test_false_offsets_missing(self):
        self._test(remove_props=['false_easting', 'false_northing'])

    def test_false_offsets_none(self):
        self._test(replace_props={'false_easting':None, 'false_northing':None})


if __name__ == "__main__":
    tests.main()
